<?php
namespace Teknisa\FIS\Modules\Acessorias\Services;

use Teknisa\FIS\Modules\Acessorias\Factories\Fis07800DacAlFactory;
use Teknisa\FIS\Modules\Acessorias\Services\Fis91000ObrigacoesAcessoriasService;
use Teknisa\FIS\Modules\Acessorias\Services\Agendamento;

use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\NewCode;
use \Teknisa\Libs\Util\Libraries\PHPMailer;

class Fis07800DacAlService {

    protected $environment;
    protected $dacAlFactory;
    protected $obrigacoesService;

    public $mensagemBody = '';
    public $mensagemBodySucess = '';
    public $mensagemError = '';
    public $mensagemErrorBody = '';
    public $geraArquivo = '';

    public function __construct(Environment $environment, Fis07800DacAlFactory $dacAlFactory, Fis91000ObrigacoesAcessoriasService $obrigacoesService,
                                    NewCode $newCode, Agendamento $agendamento ) 
    {
        $this->environment              = $environment;
        $this->dacAlFactory             = $dacAlFactory;
        $this->obrigacoesService        = $obrigacoesService;
        $this->newCode                  = $newCode;
        $this->agendamento              = $agendamento;
    }

    /*
     * Função para criar o arquivo da exportação DAC-AL.
     */
    private function createFileDacAl($row) {
        // Cria os parâmetros para buscar os dados gerados na procedure.
        $params = array(
            "NRORG" => $row["NRORG"],
            "NRSEQARQMAG" => $row["NRSEQARQMAG"]
        );
        $dacAlData = $this->dacAlFactory->getDacAlFiscalData($params);
        
        if(isset($row['NRAGENDAMENTO']) && empty($dacAlData)){
            $this->mensagemError = '<br />'."Não há dados a serem exibidos.";
            return $fileName = false;
        }
        // Chama a função para a criação do arquivo.
        $fileName = $this->obrigacoesService->createFileExportacao($row["NMARQUIVO"], $dacAlData, "DSREGARQMAG");

        // Retorna o nome do arquivo exportado.
        return $fileName;
    }

    /*
     * Função para a exportação DAC-AL Fiscal.
     */
    public function exportarDacAl($row) {
        // Valida os parâmetros associados.
        $this->validateDacAl($row);

        // Gera o código sequencial do arquivo magnético através do NOVOCODIGO.
        $row["NRSEQARQMAG"] = $this->obrigacoesService->getNextNrseqarqmag($row["NRORG"]);

        // Monta o array com os parâmetros para chamar a procedure.
        $params = array(
            "CDIMPOSTO"     => "00",
            "CDEMPRESA"     => $row["CDEMPRESA"],
            "CDINSCESTA"    => $row["CDINSCESTA"],
            "DTINICIAL"     => $row["DTINICIAL"],
            "DTFINAL"       => $row["DTFINAL"],
            "VERSAO"        => $row["VERSAO"],
            "RELEASE"       => $row["RELEASE"],
            "SBRELEASE"     => $row["SBRELEASE"],
            "TPDEC"         => $row["TPDEC"],
            "PDEC"          => $row["PDEC"],
            "INDMOV"        => $row["INDMOV"],
            "INDENC"        => $row["INDENC"],
            "GUIA"          => $row["GUIA"],
            "APURAC"        => $row["APURAC"],
            "SISTEMA"       => $row["SISTEMA"],
            "CALC"          => (empty($row["CALC"] == "") || isset($row["CALC"])) ? null : $row["CALC"],
            "GRUPPROD"      => (empty($row["CDGRUPPROD"]) || isset($row["CDGRUPPROD"])) ? null : $row["CDGRUPPROD"],
            "AIDF"          => (empty($row["AIDF"]) || isset($row["AIDF"])) ? 0 : $row["AIDF"],
            "REGANUAL"      => $row["REGANUAL"],
            "NRSEQARQMAG"   => $row["NRSEQARQMAG"]
        );

        // Chama a procedure de exportação.
        $this->dacAlFactory->callDacAlFiscalProcedure($params);

        // Foi evitado utilizar um bloco finally devido a possíveis
        // incompatibilidades com a versão do PHP.
        try{
            // Seta os parâmetros para exclusão dos dados temporários.
            $params = array(
                "NRORG" => $row["NRORG"],
                "NRSEQARQMAG" => $row["NRSEQARQMAG"]
            );
            // Chama a função para gerar o arquivo texto.
            $fileName = $this->createFileDacAl($row);

            // Exclui os dados temporários da exportação no banco de dados.
            $this->dacAlFactory->deleteDacAlFiscalData($params);

            // Retorna o nome do arquivo com suas possíveis modificações.
            return $fileName;
        } catch(\Exception $e) {
            // Exclui os dados temporários da exportação no banco de dados.
            $this->dacAlFactory->deleteDacAlFiscalData($params);
            throw $e;
        }
    }

    /*
     * Função para validação dos dados informados para o DAC-AL.
     */
    private function validateDacAl($row) {
        if (isset($row['NRAGENDAMENTO'])){
            
            $this->mensagemError = '';
            
            //var_dump($row["RELEASE"], $row["SBRELEASE"]);
            
            // Verifica se o campo "Empresa" possui algum valor informado.
            if (empty($row["CDEMPRESA"])) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Empresa deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Empresa deve ser informada.");
            }
            // Verifica se o campo "Referência" possui algum valor informado.
            if (empty($row["DTREFINI"])) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Referência deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Referência deve ser informada.");
            }
            // Verifica se o campo "Nome do Arquivo" possui algum valor informado.
            if (empty($row["NMARQUIVO"])) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Nome do Arquivo deve ser informado.';
                $this->geraArquivo = false;
                //throw new \Exception("Nome do Arquivo deve ser informado.");
            }
            // Verifica se o campo "Inscrição Estadual" possui algum valor informado.
            if (empty($row["CDINSCESTA"])) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Inscrição deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Inscrição deve ser informada.");
            }
            // Verifica se o campo "Versão" possui algum valor informado.
            if (empty($row["VERSAO"])) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Versão deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Versão deve ser informada.");
            }
            // Verifica se o campo "Release" possui algum valor informado.
            if (empty($row["RELEASE"]) && !(isset($row["RELEASE"]) && $row["RELEASE"] == "0")) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Release deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Release deve ser informada.");
            }
            // Verifica se o campo "Sub-Release" possui algum valor informado.
            if (empty($row["SBRELEASE"]) && !(isset($row["SBRELEASE"]) && $row["SBRELEASE"] == "0")) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Sub-Release deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Sub-Release deve ser informada.");
            }
            // Verifica se o campo "Tipo de Declaração" possui algum valor informado.
            if (empty($row["TPDEC"]) && !(isset($row["TPDEC"]) && $row["TPDEC"] == "0")) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Tipo de Declaração deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Tipo de Declaração deve ser informada.");
            }
            // Verifica se o campo "Período de Declaração" possui algum valor informado.
            if (empty($row["PDEC"]) && !(isset($row["PDEC"]) && $row["PDEC"] == "0")) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Período de Declaração deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Período de Declaração deve ser informada.");
            }
            // Verifica se o campo "Indicador de Movimento" possui algum valor informado.
            if (empty($row["INDMOV"]) && !(isset($row["INDMOV"]) && $row["INDMOV"] == "0")) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Indicador de Movimento deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Indicador de Movimento deve ser informada.");
            }
            // Verifica se o campo "Indicador de Encerramento" possui algum valor informado.
            if (empty($row["INDENC"]) && !(isset($row["INDENC"]) && $row["INDENC"] == "0")) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Indicador de Encerramento deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Indicador de Encerramento deve ser informada.");
            }
            // Verifica se o campo "Guia Utilizada" possui algum valor informado.
            if (empty($row["GUIA"]) && !(isset($row["GUIA"]) && $row["GUIA"] == "0")) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Guia Utilizada deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Guia Utilizada deve ser informada.");
            }
            // Verifica se o campo "Apuração Lucro Bruto" possui algum valor informado.
            if (empty($row["APURAC"]) && !(isset($row["APURAC"]) && $row["APURAC"] == "0")) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Apuração Lucro Bruto deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Apuração Lucro Bruto deve ser informada.");
            }
            // Verifica se o campo "Grupo de Produto" possui algum valor informado.
            //if (empty($row["GRUPPROD"])) {
            //    throw new \Exception("Grupo de Produto deve ser informada.");
            //}
    
    
            // Busca e atribui os valores do NRORG e datas inicial e final do mês
            // à variável row.
            
            $datas = $this->obrigacoesService->getFirstAndLastDays($row["DTREFINI"]);
            $row["DTINICIAL"] = $datas["DTINICIAL"];
            $row["DTFINAL"] = $datas["DTFINAL"];
            
            // Seta o imposto a ser validado no mapa de apuração.
            $impostos = array('00');
    
            // Valida o mapa de apuração.
            $mapaApuracIsValid = $this->obrigacoesService->mapaApuracIsValid($row, $impostos);
    
            // Verifica se o mapa está inválido, lançando um erro em caso positivo.
            if (!$mapaApuracIsValid) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Todos os mapas devem estar bloqueados para geração do arquivo.';
                $this->geraArquivo = false;
                //throw new \Exception("Todos os mapas devem estar bloqueados para geração do arquivo.");
            }

        } else {
            // Verifica se o campo "Empresa" possui algum valor informado.
            if (empty($row["CDEMPRESA"])) {
                throw new \Exception("Empresa deve ser informada.");
            }
            // Verifica se o campo "Referência" possui algum valor informado.
            if (empty($row["DTREFINI"])) {
                throw new \Exception("Referência deve ser informada.");
            }
            // Verifica se o campo "Nome do Arquivo" possui algum valor informado.
            if (empty($row["NMARQUIVO"])) {
                throw new \Exception("Nome do Arquivo deve ser informado.");
            }
            // Verifica se o campo "Inscrição Estadual" possui algum valor informado.
            if (empty($row["CDINSCESTA"])) {
                throw new \Exception("Inscrição deve ser informada.");
            }
            // Verifica se o campo "Versão" possui algum valor informado.
            if (empty($row["VERSAO"])) {
                throw new \Exception("Versão deve ser informada.");
            }
            // Verifica se o campo "Release" possui algum valor informado.
            if (empty($row["RELEASE"]) && !(isset($row["RELEASE"]) && $row["RELEASE"] == "0")) {
                throw new \Exception("Release deve ser informada.");
            }
            // Verifica se o campo "Sub-Release" possui algum valor informado.
            if (empty($row["SBRELEASE"]) && !(isset($row["SBRELEASE"]) && $row["SBRELEASE"] == "0")) {
                throw new \Exception("Sub-Release deve ser informada.");
            }
            // Verifica se o campo "Tipo de Declaração" possui algum valor informado.
            if (empty($row["TPDEC"]) && !(isset($row["TPDEC"]) && $row["TPDEC"] == "0")) {
                throw new \Exception("Tipo de Declaração deve ser informada.");
            }
            // Verifica se o campo "Período de Declaração" possui algum valor informado.
            if (empty($row["PDEC"]) && !(isset($row["PDEC"]) && $row["PDEC"] == "0")) {
                throw new \Exception("Período de Declaração deve ser informada.");
            }
            // Verifica se o campo "Indicador de Movimento" possui algum valor informado.
            if (empty($row["INDMOV"]) && !(isset($row["INDMOV"]) && $row["INDMOV"] == "0")) {
                throw new \Exception("Indicador de Movimento deve ser informada.");
            }
            // Verifica se o campo "Indicador de Encerramento" possui algum valor informado.
            if (empty($row["INDENC"]) && !(isset($row["INDENC"]) && $row["INDENC"] == "0")) {
                throw new \Exception("Indicador de Encerramento deve ser informada.");
            }
            // Verifica se o campo "Guia Utilizada" possui algum valor informado.
            if (empty($row["GUIA"]) && !(isset($row["GUIA"]) && $row["GUIA"] == "0")) {
                throw new \Exception("Guia Utilizada deve ser informada.");
            }
            // Verifica se o campo "Apuração Lucro Bruto" possui algum valor informado.
            if (empty($row["APURAC"]) && !(isset($row["APURAC"]) && $row["APURAC"] == "0")) {
                throw new \Exception("Apuração Lucro Bruto deve ser informada.");
            }
            // Verifica se o campo "Grupo de Produto" possui algum valor informado.
            //if (empty($row["GRUPPROD"])) {
            //    throw new \Exception("Grupo de Produto deve ser informada.");
            //}
    
    
            // Busca e atribui os valores do NRORG e datas inicial e final do mês
            // à variável row.
            $nrorg = $this->environment->getNrorg();
            $datas = $this->obrigacoesService->getFirstAndLastDays($row["DTREFINI"]);
            $row["DTINICIAL"] = $datas["DTINICIAL"];
            $row["DTFINAL"] = $datas["DTFINAL"];
            $row["NRORG"] = $nrorg;
    
            // Seta o imposto a ser validado no mapa de apuração.
            $impostos = array('00');
    
            // Valida o mapa de apuração.
            $mapaApuracIsValid = $this->obrigacoesService->mapaApuracIsValid($row, $impostos);
    
            // Verifica se o mapa está inválido, lançando um erro em caso positivo.
            if (!$mapaApuracIsValid) {
                throw new \Exception("Todos os mapas devem estar bloqueados para geração do arquivo.");
            }
        }
    }
    
    public function findAgendamento() {
        // Busca os parâmetros conforme empresa e nrorg.
        $agendamento = $this->dacAlFactory->findAgendamento();

        return $agendamento;
    }
    
    public function saveAgendamento($row) {
        try {
            
            $row['CDTELA'] = 'FIS07800';
            
            $dtinicial  = DateUtil::getPrimeiroDiaDoMes( $this->formataData($row['DTREFINI']));
            $dtfinal    = DateUtil::getUltimoDiaDoMes( $this->formataData($row['DTREFINI']));
            $dtinicial = $dtinicial->format("d/m/Y");
            $dtfinal = $dtfinal->format("d/m/Y");
    
            $row["DTINICIAL"] = $dtinicial;
            $row["DTFINAL"] = $dtfinal;
            
           // var_dump($row);die;
           
            
            if(isset($row['__is_new']) && $row['__is_new'] == true){
                $row["NRAGENDAMENTO"] = $this->newCode->getCodeByTableName('NRAGENDAMENTO', $row["NRORG"], 1, 10);
            }
            
            if($row['CDINSCESTA'] == 'T'){
                $arrayInscEsta = $this->dacAlFactory->getInscEsta($row);
                $cdinscesta = '_';
                
                foreach($arrayInscEsta as $value){
                    $cdinscesta = $cdinscesta.$value['CDINSCESTA'].'_';
                }
                
                $row['CDINSCESTA'] = $cdinscesta;
            } else if(is_array($row['CDINSCESTA']) == true){
                $arrayInscEsta = $row['CDINSCESTA'];
                $cdinscesta = '_';
                
                foreach($arrayInscEsta as $value){
                    $cdinscesta = $cdinscesta.$value.'_';
                }
                
                $row['CDINSCESTA'] = $cdinscesta;
            } 
        
            if(isset($row['__is_new']) && $row['__is_new'] == true){
                $this->dacAlFactory->insertAgendamento($row);
            } else {
                $this->dacAlFactory->updateAgendamento($row);
                
                 $arquivos = $this->dacAlFactory->getAgendamentoArquivo($row);
                
                if($arquivos){
                    foreach($arquivos as $arquivo){
                        unlink($arquivo['NMPATHARQ']);
                    }
                    
                    $this->dacAlFactory->deleteAgendamentoArquivo($arquivo);
                }
                
                $idstatus = 'P';
                $this->dacAlFactory->updateAgendamentoIdStatus($row, $idstatus);
                
            }
            
            $agendamento = $this->dacAlFactory->getAgendamentoInsert($row);
            
            if($agendamento){
                
                $agendamento['NRORG'] = $row['NRORG'];
                $getConfigSMTP = $this->dacAlFactory->getConfiguraSMTP($agendamento)[0];
                if(isset($row['__is_new']) && $row['__is_new'] == true){
                    $nomeJob = $row['CDTELA'].'_'.$row['NRAGENDAMENTO'];
                } else {
                    $nomeJob = $row['CDTELA'].'_'.str_pad($row['NRAGENDAMENTO'], 10, "0", STR_PAD_LEFT);
                }
                $dataContabAuto = $row['DTAGENDAMENTO'];
                $sequencialJob = $row['NRAGENDAMENTO'];
                $nrorg = $row['NRORG'];
                $descricao = $row['CDTELA'].' - DAC - AL (Alagoas)'.'. Exportação das declarações Acessórias de forma automática na data de '.$row['DTAGENDAMENTO'];
                //Conferir caminho do diretorio do xampp
                $arqCallService = $getConfigSMTP['CALLSERVPATH'];//'C:\Teknisa\CallService.php';
                //$dirphp = str_replace('/','\',$DIRPHP);
                $dirphp = $getConfigSMTP['DIRPHP'];//'C:/xampp/php';
                $jenkinsPath = $getConfigSMTP['JENKINSPATH'];//'http://192.168.120.132:8080/';
                $execPath = '/execAgendamentoDacAl';
                $periodicidade = $row['IDPERIAGEN'];
                $jenkinsCreateItem = $getConfigSMTP['JENKCREATITEM'];
                $this->agendamento->criaJob($nomeJob, $dataContabAuto, $sequencialJob, $nrorg, $descricao, $arqCallService, $dirphp, $jenkinsPath, $execPath, $periodicidade, $jenkinsCreateItem);
                
            }
            
        } catch(\Exception $e) {
            //
            throw $e;
        }
    }
    
    public function execAgendamento($row){
        try{
            $agendamento = $this->dacAlFactory->findAgendamentoExec($row)[0];
            
            $findParamDeclAcess = $this->dacAlFactory->findParamDeclAcess($agendamento);
            
            $agendamento['CDGRUPPROD'] = $findParamDeclAcess['CDGRUPPROD'];
            $agendamento['VERSAO'] = $findParamDeclAcess['VERSAO']; 
            $agendamento['RELEASE'] = $findParamDeclAcess['RELEASE'];
            $agendamento['SBRELEASE'] = $findParamDeclAcess['SBRELEASE'];
            $agendamento['TPDEC'] = $findParamDeclAcess['TPDEC']; 
            $agendamento['PDEC'] = $findParamDeclAcess['PDEC']; 
            $agendamento['INDMOV'] = $findParamDeclAcess['INDMOV']; 
            $agendamento['INDENC'] = $findParamDeclAcess['INDENC']; 
            $agendamento['GUIA'] = $findParamDeclAcess['GUIA']; 
            $agendamento['APURAC'] = $findParamDeclAcess['APURAC'];    
            $agendamento['SISTEMA'] = $findParamDeclAcess['SISTEMA']; 
            $agendamento['CALC'] = $findParamDeclAcess['CALC']; 
            $agendamento['REGANUAL'] = $findParamDeclAcess['REGANUAL'];           
            
            if($agendamento){
                $this->mensagemBody = '';
                
                $agendamento["NRORG"] = $row['NRORG'];
                
                $agendamento["DTEXECAGEND"] = $agendamento["DTATUAL"];
                
                $agendamento["DTREFINI"] = explode(' ', $agendamento["DTREFINI"])[0];
                
                $agendamento['CDINSCESTAARRAY'] = $agendamento['CDINSCESTA'];
                $arrayInscEsta = explode('_',$agendamento['CDINSCESTAARRAY']);
                
                $getConfigSMTP = $this->dacAlFactory->getConfiguraSMTP($agendamento)[0];
                $getContatos = $this->dacAlFactory->getContatos($agendamento);
                
                $mail = new PHPMailer;
                $mail->CharSet = 'UTF-8';
                $mail->isSMTP();                        // Set mailer to use SMTP
                $mail->isHTML();
                $mail->Host = $getConfigSMTP['SERVIDORSMTP'];       // Specify main and backup SMTP servers
                $mail->SMTPAuth = true;                 // Enable SMTP authentication
                $mail->Username = $getConfigSMTP['LOGINSMTP'];    // SMTP username
                $mail->Password = $getConfigSMTP['SENHASMTP'];    // SMTP password
                $mail->From = $getConfigSMTP['LOGINSMTP'];
                $mail->FromName = 'ERP Pebbian';
                $mail->Subject = 'DAC - AL de '.$agendamento['DTAGENDAMENTO'];
                // $mail->addAddress($cliente['DSEMAILCLIE']);
                // $mail->addAddress($vendedor['DSEMAILOPER']);
                
                foreach($getContatos as $contato){
                    $mail->addAddress($contato['EMAIL']);
                };
                
                $agendamento["NMEMPRESA"] = $this->dacAlFactory->getNmEmpresa($agendamento)["NMEMPRESA"];
                        
                $this->mensagemBody = 'Caro cliente, '.'<br />'.'<br />';
                $this->mensagemBodySucess = 'Arquivo de DAC - AL gerado com Sucesso!'
                .'<br />'.'Empresa: '.$agendamento['CDEMPRESA'].' - '.$agendamento['NMEMPRESA'];
                $this->mensagemErrorBody = 'Erro ao gerar o arquivo de DAC - AL:'.'<br />'.'<br />'
                .'<br />'.'Empresa: '.$agendamento['CDEMPRESA'].' - '.$agendamento['NMEMPRESA'];
                
                
                $contErro = 0;
                $contSucess = 0;
                
                foreach( $arrayInscEsta as $inscEsta ){
                    if(isset($inscEsta) && $inscEsta != ''){
                        
                       $this->geraArquivo = true;
                            
                        $dataName = str_replace('/', '', explode(' ', $agendamento["DTATUAL"])[0]);
                        $horaName = str_replace(':', '', explode(' ', $agendamento["DTATUAL"])[1]);
                        
                        if($agendamento['IDPERIAGEN'] == '0'){
                            $dtinicial = $agendamento["DTINICIAL"];
                            $dtfinal = $agendamento["DTFINAL"];
                        } else 
                        if($agendamento['IDPERIAGEN'] == '1'){
                            $dtinicial  = DateUtil::getPrimeiroDiaDoMes( $this->formataData(explode(' ', $agendamento["DTATUAL"])[0]));
                            $dtfinal    = DateUtil::getUltimoDiaDoMes( $this->formataData(explode(' ', $agendamento["DTATUAL"])[0]));
                            $dtinicial = $dtinicial->format("d/m/Y");
                            $dtfinal = $dtfinal->format("d/m/Y");
                        } else 
                        if($agendamento['IDPERIAGEN'] == '2'){
                            $dtinicial  = DateUtil::getPrimeiroDiaDoMes( $this->formataData(explode(' ', $agendamento["LAST_MONTH"])[0]));
                            $dtfinal    = DateUtil::getUltimoDiaDoMes( $this->formataData(explode(' ', $agendamento["LAST_MONTH"])[0]));
                            $dtinicial = $dtinicial->format("d/m/Y");
                            $dtfinal = $dtfinal->format("d/m/Y");
                        }
                        
                        $agendamento["DTINICIAL"] = $dtinicial;
                        $agendamento["DTFINAL"] = $dtfinal;
                        
                        $agendamento['CDINSCESTA'] = $inscEsta;
                        $agendamento["NMARQUIVO"] = $agendamento['CDTELA'].$agendamento['NRAGENDAMENTO'].'_'.$inscEsta.'_'.$dataName.$horaName;
                          
                        $fileName = $this->exportarDacAl($agendamento);
                        
                        if($this->mensagemError == ''){
                            $this->mensagemBodySucess = $this->mensagemBodySucess
                            .'<br />'.'Inscrição Estadual: '.$agendamento['CDINSCESTA'];
                            ++$contSucess;
                            
                        } else {
                            $this->mensagemErrorBody = $this->mensagemErrorBody
                            .'<br />'.'Inscrição Estadual: '.$agendamento['CDINSCESTA']
                            .$this->mensagemError.'<br />'.'<br />';
                            ++$contErro;
                        }
                        
                        if($fileName){
                            $file = sys_get_temp_dir() . DIRECTORY_SEPARATOR . $fileName;
                            $agendamento["NMPATHARQ"] = $file ;
                            
                            $verifArq = $this->dacAlFactory->verAgendamentoArquivo($agendamento);
                            
                            if(empty($verifArq)){
                                $agendamento["NRARQUIVO"] = $this->newCode->getCodeByTableName('NRARQUIVO', $agendamento["NRORG"], 1, 10);
                                $this->dacAlFactory->insertAgendamentoArquivo($agendamento);
                            }
                        }
                
                    }
                }
                
                if(($contErro > 0) && ($contSucess > 0)){
                    $mail->Body = $this->mensagemBody.'<br />'.$this->mensagemBodySucess.'<br />'.'<br />'
                    .'---------------------------------------------------------------------'.'<br />'
                    .$this->mensagemErrorBody
                    .'<br />'.'Atenciosamente,'.'<br />'.'<br />'.'Equipe ERP Pebbian.';
                    $mail->send();
                } 
                else if($contErro > 0){
                    $mail->Body = $this->mensagemBody.'<br />'
                    .$this->mensagemErrorBody
                    .'<br />'.'Atenciosamente,'.'<br />'.'<br />'.'Equipe ERP Pebbian.';
                    $mail->send();
                }
                else if($contSucess > 0){
                    $mail->Body = $this->mensagemBody.'<br />'.$this->mensagemBodySucess.'<br />'
                    .'<br />'.'Atenciosamente,'.'<br />'.'<br />'.'Equipe ERP Pebbian.';
                    $mail->send();
                }
                
                $idstatus = 'R';
                $this->dacAlFactory->updateAgendamentoIdStatus($agendamento, $idstatus);
                
            }
            
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function deleteAgendamento($row){
        try {    
            
            $agendamento = $this->dacAlFactory->getAgendamentoInsert($row)[0];
            
            
            if($agendamento){
                $nomeJob = $agendamento['CDTELA'].'_'.str_pad($agendamento['NRAGENDAMENTO'], 10, "0", STR_PAD_LEFT);
                $getConfigSMTP = $this->dacAlFactory->getConfiguraSMTP($agendamento)[0];
                
                $this->agendamento->apagaJob($nomeJob, null, $getConfigSMTP['JENKINSPATH'], $getConfigSMTP["CALLSERVPATH"]);
                
                $arquivos = $this->dacAlFactory->getAgendamentoArquivo($agendamento);
                
                if($arquivos){
                    foreach($arquivos as $arquivo){
                        unlink($arquivo['NMPATHARQ']);
                    }
                    
                    $this->dacAlFactory->deleteAgendamentoArquivo($arquivo);
                    $this->dacAlFactory->deleteAgendamento($arquivo);
                    
                } else {
                    $this->dacAlFactory->deleteAgendamento($agendamento);
                }
                
            }
            
        } catch(\Exception $e) {
            //
            throw $e;
        }
                
    }
 
    private function formataData($data) {
        if (!$data) {
            return null;
        }

        // Verifica se está com o formato de data-hora.
        if (strlen($data) == 19) {
            $data = DateUtil::truncateString($data);
        }
        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }
    
}