<?php
namespace Teknisa\FIS\Services;

use DateTime;
use Doctrine\ORM\EntityManager;
use Teknisa\FIS\Factories\Fis46000ParametrizacaoCodigosApuracaoFactory;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Row;

class Fis46000ParametrizacaoCodigosApuracaoService {

    //Mensagem padrão
    const ARQUIVO_INVALIDO = "Operação Bloqueada! O layout do arquivo não está conforme padrão estipulado pela receita federal. Favor baixar o arquivo no site do SPED Fiscal e importar novamente.";

    protected $nrorg;
    protected $operador;
    protected $currentTime;
    protected $environment;
    protected $entityManager;
    protected $newCode;
    protected $dataSourceManager;
    protected $factory;

    public function __construct(Environment $environment, EntityManager $entityManager, NewCode $newCode, $dataSourceManager, Fis46000ParametrizacaoCodigosApuracaoFactory $factory) {
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->newCode = $newCode;
        $this->dataSourceManager = $dataSourceManager;
        $this->factory = $factory;
        $this->nrorg       = $this->environment->getNrOrg();
        $this->operador    = $this->environment->getCdOperador();
        $this->currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
    }
    
    
	/*******************************************************************
	*                  Funções Referentes a aba                        *
	*                  Código da Receita                               *
	*******************************************************************/
   
    
    public function updateCodigoDaReceita($rows){
        $params = [];
        $old = $rows[0];
        $new = $rows[1];
        $params["NOVOIDDEBSCPINC"] = $new["IDDEBSCPINC"];
        $params["NOVOIDPERIODICREC"] = $new["IDPERIODICREC"];
        $params["IDDEBSCPINC"] = $old["IDDEBSCPINC"];
        $params["IDPERIODICREC"] = $old["IDPERIODICREC"];
        $params["CDVARIACAO"] = $old["CDVARIACAO"];
        $params["CDRECEITACONTR"] = $old["CDRECEITACONTR"];
        $params["CDIMPOSTO"] = $old["CDIMPOSTO"];
        $params["NRORG"] = $old["NRORG"];
        $params["DTULTATU"] = DateUtil::getDataAtual()->format('d/m/Y H:i:s');;
        $params["NRORGULTATU"] = $this->environment->getNrOrgTrab();
        $params["CDOPERULTATU"] = $this->environment->getCdoperador();
        $this->factory->updateCodreceitacontr($params);
    }

    public function validarCodigoDaReceita($row){
        $params = [];
        $params["iddebscpinc"] = $row["IDDEBSCPINC"];
        $params["idperiodicrec"] = $row["IDPERIODICREC"];
        $params["cdvariacao"] = $row["CDVARIACAO"];
        $params["cdreceitacontr"] = $row["CDRECEITACONTR"];
        $params["cdimposto"] = $row["CDIMPOSTO"];
        $params["nrorg"] = $this->environment->getNrOrgTrab();
        $codreceitacontr = $this->factory->getCodreceitacontr($params);
        if($codreceitacontr != null){
            throw new \Exception("Impossível salvar. Registro já existe.");
        }
    }

    public function isKeepingOldCodigoDaReceita($rows){
        $old = $rows[0];
        $new = $rows[1];
        return $new["IDDEBSCPINC"] == $old["IDDEBSCPINC"] &&
                $new["IDPERIODICREC"] == $old["IDPERIODICREC"] &&
                $new["CDVARIACAO"] == $old["CDVARIACAO"] &&
                $new["CDRECEITACONTR"] == $old["CDRECEITACONTR"] &&
                $new["CDIMPOSTO"] == $old["CDIMPOSTO"] &&
                $new["NRORG"] == $old["NRORG"];
    }
    
    public function salvarCodigoReceitaEstadual($row){
        try {
            $arrayCodigo  = [];
            if($row['__is_new']){
                $nrseqcodreceest = $this->newCode->getCodeByTableName('CODRECEITAEST', $this->nrorg );
                $row['NRSEQCODRECEEST'] = $nrseqcodreceest;
            }

            $this->validaDatas($row,'NATUREZA');

            $arrayCodigo [] = $row;
            $dataSet = new DataSet('codreceitaest', $arrayCodigo);
            $this->dataSourceManager->persist($dataSet);

        }catch (\Exception $e) {
            throw $e;
        }
    }
    
     // le cada linha do arquivo de tabelas e retorna os campos que estão nele informados.
    public function retornaCamposCodigoReceita ($row,$rowImportacao){
        try {
            $campos = explode("|", $row);
            $this->validaArquivoCodigoReceita($campos);

            $campos[2]= $this->criaData($campos[2]);
            //se existir data final (campo não obrigatório), formata a data informada.
            if($campos[3]){
               $campos[3]= $this->criaData($campos[3]);
            }
            return $campos;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function montaDataSetCodigoReceita ($arquivo,$rowImportacao){
        try {
            $arrayCodigo  = [];
            for ($i = 0; $i<count($arquivo); $i++){
                $dados = $this->retornaCamposCodigoReceita($arquivo[$i],$rowImportacao);
                $row = new Row();
                $row->set('__is_new',true);
                $row->set('NRORG',$this->nrorg);
                $row->set('NRORGULTATU',$this->nrorg);
                $row->set('NRORGINCLUSAO',$this->nrorg);
                $row->set('DTULTATU',$this->currentTime);
                $row->set('DTINCLUSAO',$this->currentTime);
                $row->set('CDOPERINCLUSAO',$this->operador);
                $row->set('CDOPERULTATU',$this->operador);
                $row->set('IDATIVO','S');
                $row->set('CDOPERADOR',$this->operador);
                $row->set('CDRECEITAEST',$dados[0]);
                $row->set('NMRECEITAEST',$dados[1]);
                $row->set('DTINIVALID',$dados[2]);
                $row->set('DTFIMVALID',$dados[3]);
                $row->set('NRSEQCODRECEEST', $this->newCode->getCodeByTableName('CODRECEITAEST', $this->nrorg) );
                $row->set('SGESTADO',$rowImportacao['SGESTADO']);
                $row->set('CDPAIS','0055');
                $arrayCodigo [] = $row;
            }
            $dataSet = new DataSet('codreceitaest', $arrayCodigo);
            return $dataSet;
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaArquivoCodigoReceita ($campos){
         try {
            //verifica se o arquivo possui os quatros campos necessarios;
            if(count($campos)!==4){
                throw new \Exception(self::ARQUIVO_INVALIDO, 1);
            }else{
                //verifica se os campos obrigatorios estao preenchidos
                if( !($campos[0] && $campos[1] && $campos[2]) ){
                    throw new \Exception(self::ARQUIVO_INVALIDO, 1);
                }
            
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }



    /********************************************************************
	*                  Funções Referentes a aba                         *
	*                  Código de Ajuste                                 *
	********************************************************************/
	
	
    public function saveCodigoAjuste($row){
        try {
            $arrayCodigo  = [];
            if($row['__is_new']){
                $nrseqcodajuste  =  $this->newCode->getCodeByTableName('CODAJUSTEEFDICMS', $this->nrorg) ;
                $row['NRSEQCODAJUSTE'] = $nrseqcodajuste;
            }

            $this->validaDatas($row,'AJUSTE');

            $arrayCodigo [] = $row;
            $dataSet = new DataSet('codajusteefdicms', $arrayCodigo);
            $this->dataSourceManager->persist($dataSet);

        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function montaDataSetCodigoAjuste ($arquivo,$rowImportacao){
        try {
            $arrayCodigo  = [];
            for ($i = 0; $i<count($arquivo); $i++){
                $dados = $this->retornaCamposCodigoAjuste($arquivo[$i],$rowImportacao);
                $row = new Row();
                $row->set('__is_new',true);
                $row->set('NRORG',$this->nrorg);
                $row->set('NRORGULTATU',$this->nrorg);
                $row->set('NRORGINCLUSAO',$this->nrorg);
                $row->set('DTULTATU',$this->currentTime);
                $row->set('DTINCLUSAO',$this->currentTime);
                $row->set('CDOPERINCLUSAO',$this->operador);
                $row->set('CDOPERULTATU',$this->operador);
                $row->set('IDATIVO','S');
                $row->set('CDOPERADOR',$this->operador);
                $row->set('CDAJUSTE',$dados[0]);
                $row->set('DSAJUSTE',$dados[1]);
                $row->set('DSOBSAJUSTE',null);
                $row->set('DTINIVALID',$dados[2]);
                $row->set('DTFIMVALID',$dados[3]);
                $row->set('NRSEQCODAJUSTE', $this->newCode->getCodeByTableName('CODAJUSTEEFDICMS', $this->nrorg) );
                $row->set('IDTPCODRECEITA',$rowImportacao['IDTPCODRECEITA']);
                $row->set('IDTPAJUSTE',$rowImportacao['IDTPAJUSTE']);
                $row->set('SGESTADO',$rowImportacao['SGESTADO']);
                $row->set('CDPAIS','0055');
                $arrayCodigo [] = $row;
            }
            $dataSet = new DataSet('codajusteefdicms', $arrayCodigo);
            return $dataSet;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    // le cada linha do arquivo de tabelas e retorna os campos que estão nele informados.
    public function retornaCamposCodigoAjuste ($row,$rowImportacao){
        try {
            $campos = explode("|", $row);
            $this->validaArquivo($campos,$rowImportacao);

            $campos[2]= $this->criaData($campos[2]);
            //se existir data final (campo não obrigatório), formata a data informada.
            if($campos[3]){
               $campos[3]= $this->criaData($campos[3]);
            }
            return $campos;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    //valida cada linha do arquivo conforme o padrão criado pela receita federal.
    public function validaArquivo ($campos,$rowImportacao){
        $etadoArquivoImportacao = substr($campos[0],0,2);
        $tipoAjuste = $rowImportacao['IDTPAJUSTE'];
        try {
            //verifica se o arquivo possui os quatros campos necessarios;
            if(count($campos)!==4){
                throw new \Exception(self::ARQUIVO_INVALIDO, 1);
            }else{
                //verifica se os campos obrigatorios estao preenchidos
                if( !($campos[0] && $campos[1] && $campos[2]) ){
                    throw new \Exception(self::ARQUIVO_INVALIDO, 1);
                }
                else{
                    //verifica se o estado de importação do arquivo e o mesmo do qual ele deseja realizar a imortacao
                    if( $etadoArquivoImportacao !== $rowImportacao['SGESTADO'] ){
                        throw new \Exception(self::ARQUIVO_INVALIDO, 1);
                    }else{
                        //verifica se o codigo de ajuste e valido para o tipo de importação escolhido.
                        if($tipoAjuste =='D'){
                            if( strlen($campos[0]) !== 10 ){
                                throw new \Exception(self::ARQUIVO_INVALIDO, 1);
                            }
                        }
                        //verifica se o codigo de ajuste e valido para o tipo de importação escolhido.
                        else if($tipoAjuste =='A'){
                            if( strlen($campos[0]) !== 8 ){
                                throw new \Exception(self::ARQUIVO_INVALIDO, 1);
                            }
                        }
                    }
                }

            }
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function montaDataSetCodigoAjusteFederal($arquivo,$rowImportacao){
        try {
            $arrayCodigo  = [];
            for ($i = 0; $i<count($arquivo); $i++){
                $dados = $this->retornaCamposCodigoReceita($arquivo[$i],$rowImportacao);
                $row = new Row();
                $row->set('__is_new',true);
                $row->set('NRORG',$this->nrorg);
                $row->set('NRORGULTATU',$this->nrorg);
                $row->set('NRORGINCLUSAO',$this->nrorg);
                $row->set('DTULTATU',$this->currentTime);
                $row->set('DTINCLUSAO',$this->currentTime);
                $row->set('CDOPERINCLUSAO',$this->operador);
                $row->set('CDOPERULTATU',$this->operador);
                $row->set('IDATIVO','S');
                $row->set('CDOPERADOR',$this->operador);
                $row->set('CDAJUSTE',$dados[0]);
                $row->set('DSAJUSTE',$dados[1]);
                $row->set('DSOBSAJUSTE',null);
                $row->set('DTINIVALID',$dados[2]);
                $row->set('DTFIMVALID',$dados[3]);
                $row->set('NRSEQCODAJUSTE', $this->newCode->getCodeByTableName('CODAJUSTEEFDICMS', $this->nrorg) );
                $row->set('IDTPCODRECEITA',$rowImportacao['IDTPCODRECEITA']);
                $row->set('IDTPAJUSTE','A');
                $row->set('SGESTADO',null);
                $row->set('CDPAIS',null);
                $arrayCodigo [] = $row;
            }
            $dataSet = new DataSet('codajusteefdicms', $arrayCodigo);
            return $dataSet;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    


    /*********************************************************************
	*                  Funções Gerais                                    *
	*********************************************************************/
	

	public function salvarImportacaoTabelas ($dataset){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $texto = $this->retornaArquivo($dataset['row']);
            switch ($dataset['row']['TIPO']) {
                case 'RECEITAESTADUAL':
                    $dataSet = $this->montaDataSetCodigoReceita ($texto, $dataset['row']);
                    $valorArray = count($dataset['dataset']);
                    for ($i=0; $i<$valorArray; $i++){
                        $a = $dataset['dataset'][$i]['CDRECEITAEST'];
                        $b = $dataSet->getRows();
                        $c = $b[0]['CDRECEITAEST'];
                        $valorArray2 = count($b); 
                         for ($i2 = 0 ; $i2 < $valorArray2 ; $i2++){
                            if($a == $b[$i2]['CDRECEITAEST']){
                                 $this->factory->deleteCodigoReceita($this->nrorg, $dataset['row']['SGESTADO'], $dataset['dataset'][$i]['CDRECEITAEST']);
                            }
                        }
                     }
                    $this->entityManager->flush();
                    break;
                
                case 'AJUSTEESTADUAL':
                    
                    $dataSet = $this->montaDataSetCodigoAjuste ($texto, $dataset['row']);
                    $this->factory->deleteCodigoAjusteEstadual($this->nrorg, $dataset['row']['SGESTADO'], $dataset['row']['IDTPAJUSTE']);
                    $this->entityManager->flush();
                    break;
                
                case 'AJUSTEFEDERAL':
                    
                    $dataSet = $this->montaDataSetCodigoAjusteFederal ($texto, $dataset['row']);
                    $this->factory->deleteCodigoAjusteFederal($this->nrorg);
                    $this->entityManager->flush();
                    break;
            }
            $this->dataSourceManager->persist($dataSet);
            $this->entityManager->flush();
            $connection->commit();
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
	
    public function validaDatas ($rowAtual,$tabela){
        try {
            $resul = '';            
            if($tabela==='AJUSTE')
                $resul = $this->factory->validaIntervaloDatas($rowAtual['CDAJUSTE'], $rowAtual['DTINIVALID'], $rowAtual['NRSEQCODAJUSTE'], $rowAtual['IDTPAJUSTE'], $rowAtual['SGESTADO'], $rowAtual['DTFIMVALID']);
            else if ($tabela ==='NATUREZA')
                $resul = $this->factory->validaIntervaloDatasCodigoNatureza($rowAtual['CDRECEITAEST'], $rowAtual['DTINIVALID'], $rowAtual['NRSEQCODRECEEST'], $rowAtual['SGESTADO'], $rowAtual['DTFIMVALID']);

            if(!empty($resul)){
                if( $resul[0]['ERRO'] ){
                    if($resul[0]['ERRO']=='DtFim_Inf'){
                        throw new \Exception("Data Final deve ser informada. Informação já cadastrada para outro intervalo em período posterior.", 1);
                    }else if($resul[0]['ERRO']=='DtInt_Inv'){
                        throw new \Exception("Informação já cadastrada para o intervalo de validade.", 1);
                    }else if($resul[0]['ERRO']=='DtFimOld_Inf'){
                        throw new \Exception("Informação já cadastrada para intervalo inferior sem data final informada.", 1);
                    }
                }
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function retornaArquivo ($tabela){
        try {
            $file = $tabela['TABELA'][0]['b64File'];
           // var_dump($file);die;
            $pos  = strpos($file, ',');
            $file = substr($file,$pos+1);
            $file =  base64_decode($file);
            $file = utf8_encode($file);
            $conteudo = preg_split('/\R/', $file);

            //o arquivo possui no minimo 3 linhas (primeira cabeçalho  e a ultima uma linha em branco)
            if(count($conteudo)>=3){
                array_shift($conteudo);
                array_pop($conteudo);
                return $conteudo;
            }else{
                throw new \Exception(self::ARQUIVO_INVALIDO, 1);
            }

        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function criaData ($valor){
        try {
            $data = DateTime::createFromFormat('dmY', $valor);
            $data = date_format($data, 'd/m/Y');
            return $data;
        }catch (\Exception $e) {
            throw new \Exception(self::ARQUIVO_INVALIDO, 1);
        }
    }

}